from datetime import datetime
import re
from typing import List

from diveharderapi.enums import MajorOrderTypes, RewardTypes, ValueTypes
from diveharderapi.utils import hdml_to_md


class BaseObject:
    def __init__(self, client) -> None:
        from diveharderapi.api_client import DiveHarderApiClient

        self.client: DiveHarderApiClient = client

    @classmethod
    def from_json(cls, client, json):
        return cls(client, **json)

    def __getitem__(self, item):
        return getattr(self, item)

    def __delitem__(self, item):
        delattr(self, item)

    def __setitem__(self, item, value):
        setattr(self, item, value)

    def __iter__(self):
        return iter(self.__dict__)

    def __len__(self):
        return len(self.__dict__)

    def __repr__(self):
        return f"{self.__class__.__name__}({self.__dict__})"


class Dispatch(BaseObject):

    def __init__(
        self, client, id: int, published: int, type: int, tagIds: list, message: str
    ):
        super().__init__(client)

        self.id = id
        self.published = datetime.fromtimestamp(self.client.fix_timestamp(published))
        self.type = type
        self.tagIds = tagIds
        self.message = message

    @property
    def as_plaintext(self):
        return re.sub(r"<.*?>", "", self.message)

    @property
    def as_markdown(self):
        return hdml_to_md(self.message)


class MajorOrderReward(BaseObject):

    def __init__(
        self,
        client,
        id: int,
        type: int,
        amount: int,
    ):
        super().__init__(client)

        self.id = id
        self.type = RewardTypes(type)
        self.amount = amount


class MajorOrderSettings(BaseObject):

    def __init__(
        self,
        client,
        type: int,
        title: str,
        brief: str,
        description: str,
        reward: MajorOrderReward,
    ):
        super().__init__(client)

        self.type = type
        self.title = title
        self.brief = brief
        self.description = description
        self.reward = reward


class MajorOrderTask(BaseObject):

    def __init__(
        self,
        client,
        type: int,
        values: dict,
    ):
        super().__init__(client)
        self.type = MajorOrderTypes(type)
        self.values = values


class MajorOrder(BaseObject):

    def __init__(
        self,
        client,
        id: int,
        progress: int,
        expiresIn: int,
        settings: MajorOrderSettings,
        tasks: List[MajorOrderTask],
    ):
        super().__init__(client)

        self.id = id
        self.progress = progress
        self.expiresIn = expiresIn
        self.settings = settings
        self.tasks = tasks

    @classmethod
    def from_json(cls: "MajorOrder", client, json):
        if isinstance(json, list):
            json = json[0]

        tasks = []

        for i in json.get("setting", {}).get("tasks", []):

            vals = {}
            for j, k in enumerate(i["values"]):

                try:
                    val = ValueTypes(i["valueTypes"][j])
                except ValueError:
                    val = ValueTypes.UNKNOWN
                vals[val] = k

            tasks.append(MajorOrderTask(client, type=i["type"], values=vals))

        settings = MajorOrderSettings(
            client,
            type=json["setting"]["type"],
            title=json["setting"]["overrideTitle"],
            brief=json["setting"]["overrideBrief"],
            description=json["setting"]["taskDescription"],
            reward=MajorOrderReward(
                client,
                id=json["setting"]["reward"]["id32"],
                type=json["setting"]["reward"]["type"],
                amount=json["setting"]["reward"]["amount"],
            ),
        )
        return cls(
            client=client,
            id=json["id32"],
            progress=json["progress"],
            expiresIn=json["expiresIn"],
            settings=settings,
            tasks=tasks,
        )
